"""Utils module."""


def response_to_dict(response):
    """
    Convert a `scrapy.http.Response` to a dictionnary.
    """
    return {
        'status': response.status,
        'url': response.url,
        'headers': response.headers.to_unicode_dict(),
        'body': response.text,
    }



import json
import unittest

from scrapy.http import TextResponse

from sentry_scrapy import utils


class UtilsTestCase(unittest.TestCase):

    def test_response_to_dict(self):
        resp = TextResponse(
            'https://url.com',
            status=201,
            headers={'Content-type': 'text/html'},
            body=b'<!doctype><html><body><p>test</p></body></html>'
        )
        resp_dict = utils.response_to_dict(resp)
        self.assertEqual(resp_dict, {
            'status': 201,
            'url': 'https://url.com',
            'headers': {'content-type': 'text/html'},
            'body': '<!doctype><html><body><p>test</p></body></html>',
        })
        try:
            json.dumps(resp_dict)
        except (ValueError, TypeError) as error:
            self.fail('Response dict is not JSON serializable: {}'.format(error))



"""Scrapy extension."""
from scrapy import signals
from sentry_sdk import push_scope, capture_exception
from sentry_sdk.integrations.logging import ignore_logger

from .utils import response_to_dict


class SentryExtension:
    """
    Scrapy extension that uses Sentry to log errors.
    """
    @classmethod
    def from_crawler(cls, crawler):
        """
        Initialize the extension from a crawler instance and connect
        to the `spider_error` signal.
        """
        extension = cls()
        crawler.signals.connect(extension.spider_error, signal=signals.spider_error)

        # ignore sentry logger
        ignore_logger('scrapy.core.scraper')

        return extension

    @staticmethod
    def spider_error(failure, response, spider):
        """
        This function is used as a callback to the `spider_error` signal.

        :param failure:        the exception raised
        :param response:       the response being processed when the exception was raised
        :param spider:         the spider which raised the exception
        """
        # update sentry scope on the fly
        with push_scope() as scope:
            scope.set_tag('spider', spider.name)
            scope.set_extra('response', response_to_dict(response))
            # catch the exception
            capture_exception(failure.value)



import unittest
from unittest.mock import MagicMock, patch

from scrapy import signals
from scrapy.http import TextResponse
from twisted.python.failure import Failure

from sentry_scrapy.extension import SentryExtension


class SentryExtensionTestCase(unittest.TestCase):

    @patch('sentry_scrapy.extension.ignore_logger')
    def test_from_crawler(self, ignore_logger):
        crawler = MagicMock()
        extension = SentryExtension.from_crawler(crawler)
        self.assertIsInstance(extension, SentryExtension)
        crawler.signals.connect.assert_called_with(
            extension.spider_error,
            signal=signals.spider_error
        )
        ignore_logger.assert_called_with('scrapy.core.scraper')

    @patch('sentry_scrapy.extension.capture_exception')
    @patch('sentry_sdk.scope.Scope.set_tag')
    @patch('sentry_sdk.scope.Scope.set_extra')
    def test_spider_error(self, scope_set_extra, scope_set_tag, capture_exception):
        resp = TextResponse(
            'https://url.com',
            status=201,
            headers={'Content-type': 'text/html'},
            body=b'<!doctype><html><body><p>test</p></body></html>'
        )
        error = Exception('Boom')
        failure = Failure(error)
        spider = MagicMock()
        spider.name = 'superspider'
        SentryExtension.spider_error(failure, resp, spider)

        scope_set_tag.assert_called_with('spider', 'superspider')
        scope_set_extra.assert_called_with('response', {
            'status': 201,
            'url': 'https://url.com',
            'headers': {'content-type': 'text/html'},
            'body': '<!doctype><html><body><p>test</p></body></html>',
        })
        capture_exception.assert_called_with(error)



#!/usr/bin/env python

from setuptools import setup, find_packages


VERSION = '0.1'


setup(
    name='sentry-scrapy',
    version=VERSION,
    description='Scrapy integration with Sentry SDK (unofficial)',
    author='Maxime Vdb',
    author_email='me@maxvdb.com',
    packages=find_packages(),
    install_requires=['Scrapy', 'sentry-sdk'],
    license="MIT",
    keywords="sentry scrapy sdk integration",
    url='https://github.com/m-vdb/sentry-scrapy',
    download_url='https://github.com/m-vdb/sentry-scrapy/archive/v{}.tar.gz'.format(VERSION),
    project_urls={
        "Source Code": "https://github.com/m-vdb/sentry-scrapy",
    }
)
